package net.sourceforge.webstretch.request;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import net.sourceforge.webstretch.server.ProxyConfiguration;

/**
 * Records all information in a request line
 * @author Simon Shanks
 */
public class RequestLine {
	
	public final static char PARAMETER_SEPERATOR='&';
	public final static char PARAMETER_FROM_URL_SEPERATOR='?';
	public final static char PARAMETER_VALUE_SEPERATOR='=';
	private final static int DEFAULT_HTTP_PORT=80;
	
	private static final String HTTP_PREFIX="http://";
	
	private String method;
	private String url;
	private String uri;	
	private String httpVersion;
	private String host;
	private int port;
	private HashMap parameters = new HashMap();
	private boolean isRequestLocal=false;
	
	/**
	 * Default constructor
	 */
	public RequestLine(){
	}
	
	public RequestLine(String requestLine) {

		StringTokenizer st = new StringTokenizer(requestLine);
		this.setMethod(st.nextToken());
		this.setUrl(st.nextToken());
		// if httpVersion is null it should really be set to HTTP/0.9
		if (st.hasMoreTokens()) {
			this.setHttpVersion(st.nextToken());
		} else {
			this.setHttpVersion("HTTP/0.9");
		}
		if (!this.isRequestLocal){
			URL completeURL = null;
			try {
				// https requests are stripped of https:// and :443 added (as seen in IE)
				// TODO uri and queries hid in https requests!
				if (!(this.url.startsWith(HTTP_PREFIX))) {
					this.url = HTTP_PREFIX + this.url;
				}
				completeURL = new URL(this.url);
				// set the port number
				int portFound = completeURL.getPort();
				if (-1 == portFound) {
					setPort(DEFAULT_HTTP_PORT);
				} else {
					setPort(portFound);
				}
				// TODO deal with #bookmark links
				setHost(completeURL.getHost());
				setUri(completeURL.getPath());
				// query string after ? is completeURL.getQuery()
			} catch (MalformedURLException e) {
				System.out.println("This URL caused the problem - " + this.url);
				e.printStackTrace();
			}
		}

	}
	
	/**
	 * Strips parameters sent using the GET method
	 * @param url The URL or URI containing the GET parameters
	 */
	private void stripParametersFromURL(String url) {
		int positionOfSeperator = url.indexOf(PARAMETER_FROM_URL_SEPERATOR);
		if (positionOfSeperator == -1) {
			// there are no parameters being sent
			return;
		}
		int positionOfStartOfParams=++positionOfSeperator;
		if ((positionOfStartOfParams) >= url.length())
			return;
		String allParameters =
			url.substring(positionOfStartOfParams, url.length());
		String parameterPair = null;
		String parameterName = null;
		String parameterValue = null;
		int positionOfEqualSign = 0;
		int positionOfParamValue = 0;
		StringTokenizer st = new StringTokenizer(allParameters, String.valueOf(PARAMETER_SEPERATOR));
		while (st.hasMoreTokens()) {
			parameterPair = st.nextToken();
			positionOfEqualSign = parameterPair.indexOf(PARAMETER_VALUE_SEPERATOR);
			if (positionOfEqualSign == -1)
				continue;
			parameterName = parameterPair.substring(0, positionOfEqualSign);
			positionOfParamValue = ++positionOfEqualSign;
			if (positionOfParamValue < parameterPair.length()) {
				parameterValue =
					URLDecoder.decode(
						parameterPair.substring(
							positionOfParamValue,
							parameterPair.length()));
			} else {
				parameterValue = null;
			}
			this.setParameter(parameterName, parameterValue);
		}
	}
	
	/**
	 * Gets a list of all the parameter names sent in a request
	 * @return An array of parameter names
	 */
	public String[] getParameterNames() {
		Collection parameterCollection = parameters.keySet();
		String[] parameterNames = new String[parameterCollection.size()];
		Iterator iterator = parameterCollection.iterator();
		int i = 0;
		while (iterator.hasNext()) {
			parameterNames[i] = (String) iterator.next();
			i++;
		}
		return parameterNames;
	}
	
	/**
	 * Resolves the parameter value for a given parameter name in a request
	 * @param parameterName The name of the parameter that contains a value
	 * @return The value associated with the parameter name
	 */
	public String getParameterValue(String parameterName) {
		return (String) parameters.get(parameterName);
	}
	
	/**
	 * @return
	 */
	public String getHttpVersion() {
		return httpVersion;
	}

	/**
	 * @return
	 */
	public String getMethod() {
		return method;
	}

	/**
	 * @return
	 */
	public String getUri() {
		return this.uri;
	}

	/**
	 * @return
	 */
	public String getUrl() {
		return this.url;
	}

	/**
	 * @param string
	 */
	public void setHttpVersion(String httpVersion) {
		this.httpVersion = httpVersion;
	}

	/**
	 * @param string
	 */
	public void setMethod(String method) {
		this.method = method;
	}

	/**
	 * @param string
	 */
	public void setUrl(String url) {
		this.stripParametersFromURL(url);
		if ((url != null)&& (url.startsWith("http://webstretch"))){
			this.url = url;
			isRequestLocal=true;
			this.host=ProxyConfiguration.PROXY_HOST;
			this.port=ProxyConfiguration.PROXY_PORT;
			String proxyURL = "http://webstretch";
			this.setUri(this.url.substring(proxyURL.length()));
		}
		else{
			isRequestLocal=false;
			// https requests are stripped of https:// and :443 added (as seen in IE)
			// TODO uri and queries hid in https requests!
			if (url.startsWith(HTTP_PREFIX)) 
				this.url = url;
			else
				this.url = HTTP_PREFIX + this.url;
		}
	}

	/**
	 * Sets a parameter pair. Any old values with the same name will be replaced.
	 * @param name The name of the parameter to store.
	 * @param value The value to associate with the parameter name.
	 */
	public void setParameter(String name, String value) {
		this.parameters.put(name, value);
	}
	
	public boolean isRequestLocal(){
		return isRequestLocal;
	}
	
	/**
	 * Generates a legal request line 
	 * @return Request line with any request parameters attached (GET request)
	 */
	public String toString() {
		StringBuffer generatedRequest=new StringBuffer(this.getMethod());
		generatedRequest.append(" ");
		// setup the request line
		if (ProxyConfiguration.isProxyDefined()){
			generatedRequest.append(HTTP_PREFIX);
			generatedRequest.append(this.getHost());
		}
		generatedRequest.append(this.getUri());
			
		String parameterNames[]=this.getParameterNames();
		String parameterName;
		String parameterValue;
		// loop through all parameters and add to url
		for (int i = 0; i < parameterNames.length; i++) {
			if (i==0)
				generatedRequest.append(String.valueOf(RequestLine.PARAMETER_FROM_URL_SEPERATOR));
			else
				generatedRequest.append(String.valueOf(RequestLine.PARAMETER_SEPERATOR));
			parameterName=parameterNames[i];
			parameterValue=this.getParameterValue(parameterName);
			generatedRequest.append(parameterName);
			generatedRequest.append(String.valueOf(RequestLine.PARAMETER_VALUE_SEPERATOR));
			if (parameterValue!=null && parameterValue.length()>0)
				generatedRequest.append(URLEncoder.encode(parameterValue));
		}
		generatedRequest.append(" ");
		generatedRequest.append(this.getHttpVersion());
		generatedRequest.append("\r\n");
		return generatedRequest.toString();
	}

	/**
	 * @return
	 */
	public String getHost() {
		return this.host;
	}

	/**
	 * @return
	 */
	public int getPort() {
		return this.port;
	}

	/**
	 * @param string
	 */
	public void setHost(String host) {
		this.host = host;
	}

	/**
	 * @param i
	 */
	public void setPort(int port) {
		this.port = port;
	}

	/**
	 * @param string
	 */
	public void setUri(String uri) {
		this.uri = uri;
	}

}
