package net.sourceforge.webstretch.response;

import java.io.*;
import java.net.Socket;
import java.net.UnknownHostException;
import net.sourceforge.webstretch.server.Statistics;

import net.sourceforge.webstretch.request.RequestBody;
import net.sourceforge.webstretch.request.RequestHeaders;
import net.sourceforge.webstretch.request.RequestLine;
import net.sourceforge.webstretch.server.ProxyConfiguration;

/**
 * Gets a web resource outside of the proxy
 * @author Simon Shanks
 */
public class ExternalResponse extends AbstractHTMLResponse {
	
	private static final int BUFFER_SIZE=512;
	
	private RequestLine requestLine;
	private RequestHeaders requestHeaders;
	private byte[] requestBody;
	
	public ExternalResponse (RequestLine requestLine){
		this.requestLine=requestLine;
	}
	/**
	 * 
	 * @param requestLine
	 * @param requestHeaders
	 * @param requestBody
	 */
	public ExternalResponse (RequestLine requestLine,RequestHeaders requestHeaders,byte[] requestBody){
		this.requestLine=requestLine;
		this.requestHeaders=requestHeaders;
		this.requestBody=requestBody;
	}
	
	/**
	 * Set up all request information that has been configured via the
	 * webstretch interface which provides alteration of the initial request
	 * @param alteredRequestBody The information in the POST request
	 * generated by a form on the webstretch page
	 */
	public ExternalResponse (byte[] alteredRequestBody){
		requestLine=new RequestLine();
		requestHeaders=new RequestHeaders();
		RequestBody requestBody=new RequestBody(alteredRequestBody);
		String[] parameterNames=requestBody.getParameterNames();
		for (int i=0;i<parameterNames.length;i++) {
			String elementName=parameterNames[i];
			String elementValue=requestBody.getParameterValue(elementName);
			// each item submitted in the form has a prefix to determine its purpose
			if (elementName.startsWith("param")) {
				String parameterName = elementName.substring("param".length());
				requestLine.setParameter(parameterName, elementValue);
				continue;
			}
			if (elementName.startsWith("head")) {
				String headerName = elementName.substring("head".length());
				requestHeaders.setHeader(headerName, elementValue);
				continue;
			}
			if (elementName.compareTo("method") == 0) {
				requestLine.setMethod(elementValue);
				continue;
			}
			if (elementName.compareTo("port") == 0) {
				requestLine.setPort(Integer.parseInt(elementValue));
				continue;
			}
			if (elementName.compareTo("uri") == 0) {
				requestLine.setUri(elementValue);
				continue;
			}
			if (elementName.compareTo("version") == 0) {
				requestLine.setHttpVersion(elementValue);
				continue;
			}
			if (elementName.compareTo("host") == 0) {
				requestLine.setHost(elementValue);
				continue;
			}
			if (elementName.compareTo("body") == 0) {
				// TODO set body
				if (elementValue!=null && elementValue.length()!=0)
					this.requestBody=elementValue.getBytes();
				continue;
			}
		}
	}

	/* (non-Javadoc)
	 * @see net.sourceforge.webstretch.server.ProxyResponse#sendResultsToClient(java.io.BufferedOutputStream)
	 */
	public void sendResultsToClient(BufferedOutputStream out)
		throws UnknownHostException,FileNotFoundException {
		try{
		
			ResponsePayload responsePayload=this.getExternal();
			if (showTotalResponse(requestLine.getMethod()))
				// wrap the response in HTML so everything can be seen in the browser
				getTotalResponse(responsePayload,out);
			else{
				// get the response back from the external resource as normal
				out.write(responsePayload.getResponseData());
				out.flush();
				{
					Object[] comments=responsePayload.getComments();
					for (int i=0;i<comments.length;i++){
						System.out.println("=== Comment in "+requestLine.getUri()+" ===");
						System.out.println((String)comments[i]);
					}		
				}
			}
		
		} catch (UnknownHostException e) {
			throw e;
		} catch (FileNotFoundException e) {
			throw e;
		} catch (IOException e) {
			e.printStackTrace();
		} 
		
	}
	
	public ResponsePayload getExternal()
		throws UnknownHostException,FileNotFoundException {
			Socket soc = null;
			BufferedInputStream serverInput = null;
			BufferedOutputStream serverOutput = null;
			ResponsePayload responsePayload=null;
			try {
				// setup the network connections
				if (ProxyConfiguration.isProxyDefined())
					soc = new Socket(ProxyConfiguration.getExternalProxyHost(),ProxyConfiguration.getExternalProxyPort());
				else
					soc = new Socket(requestLine.getHost(), requestLine.getPort());

				serverOutput = new BufferedOutputStream(soc.getOutputStream());
				serverInput = new BufferedInputStream(soc.getInputStream());

				serverOutput.write(requestLine.toString().getBytes());
				
				if (requestBody!=null && requestHeaders!=null){
					requestHeaders.setHeader("Content-Length",Integer.toString(requestBody.length));
				}
			
				// add list of headers to request	
				if (requestHeaders!=null)
					serverOutput.write(requestHeaders.toString().getBytes());		
				serverOutput.write(CRLF.getBytes());
				serverOutput.flush();

				// add body if there is any body data			
				if (requestBody!=null){
					serverOutput.write(requestBody);
					serverOutput.flush();
				}		
				
				responsePayload=getExternalResource(serverInput);
				
			} catch (UnknownHostException e) {
				throw e;
			} catch (FileNotFoundException e) {
				throw e;
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				// close used network connections
				try {
					if (serverInput!=null)		
						serverInput.close();
					if (serverOutput!=null)
						serverOutput.close();		
					if (soc!=null)
						soc.close();
				} catch (Exception e){
					e.printStackTrace();
				}			
			}
			return responsePayload;
	}
	
	private void getTotalResponse(ResponsePayload responsePayload,BufferedOutputStream out) throws IOException{
			
		StringBuffer htmlContent = new StringBuffer();
		htmlContent.append(getStartOfHTML("WebStretch External Response"));
		htmlContent.append("<p><pre>");
		htmlContent.append(responsePayload.toString());
		htmlContent.append("</pre></p>");
		htmlContent.append("<a href=\"javascript:history.back()\">&lt;&lt;&nbsp;Back</a>");
		htmlContent.append(getEndOfHTML());
		
		{
			// send header details		
			String headerContent=getHeaderResponse(htmlContent.toString().getBytes().length);
			out.write(headerContent.getBytes());		
		}
		this.closeHeader(out);
		
		out.write(htmlContent.toString().getBytes());
		out.flush();
	}
	
	/**
	 * Transfers data coming in via the inputstream and passes it onto the output stream
	 * @param in
	 * @param out
	 * @throws IOException
	 */
	private ResponsePayload getExternalResource(BufferedInputStream in) throws FileNotFoundException,IOException{
		long totalBytes=0;
		ResponsePayload responsePayload=new ResponsePayload();
		
		try {
			byte[] buffer=new byte[BUFFER_SIZE];
			int bytesRead=0;
			while (true){
				bytesRead=in.read(buffer);
				if (bytesRead==-1)
					break;
				responsePayload.addResponseData(buffer,bytesRead);
				totalBytes+=bytesRead;
			}
		} catch (IOException e){
			// missing resource
			System.err.print("Error trying to get "+requestLine);
			System.err.println("File may not exist or connection to down external server down");
			// TODO deal with not find a file
			//throw new FileNotFoundException("File may not exist or connection down");
		}
		
		Statistics.addToTotalBytesReceived(requestLine.getHost(),totalBytes);	
		return responsePayload;
	}
	
	/**
	 * Tests whether to show what the external source has responded with, rather
	 * than just pass it back to the client
	 * @param method The HTTP method
	 * @return false if the request is to passed directly back to the client
	 */
	private boolean showTotalResponse(String method){
		if (method == null)
			return false;
		if (0== method.compareToIgnoreCase("HEAD") || 0== method.compareToIgnoreCase("OPTIONS") || 0== method.compareToIgnoreCase("TRACE"))
			return true;
		return false;
	}
}
