package net.sourceforge.webstretch.server;

import java.io.*;
import java.net.*;

import net.sourceforge.webstretch.request.*;

/**
 * Detects whether the requested resource should create a page showing 
 * the request details. It shouldnt be created for things like the
 * images attached to web pages.
 * @author Simon Shanks
 */
public class InspectionDecision {

	public static boolean inspectionActivated=true;
	private boolean isValidInspectionResource = true;

	/**
	 * Decides whether a resource request is valid for inspection.
	 * Items like images, which are part of a page request, are resources
	 * which should not be intercepted and generate inspection pages.
	 * @param requestLine Details of the current request
	 */
	public InspectionDecision(RequestLine requestLine,boolean containsAlertWord) {
		isValidInspectionResource = true;
		
		// test to see if inspection has been turned off and does contain alert word
		if (!inspectionActivated && !containsAlertWord){
			isValidInspectionResource=false;
			return;		
		}		
		// use the method to see if inspection should take place
		{
			String method = requestLine.getMethod();
			if (method == null){
				isValidInspectionResource=false;
				return;
			}
			// only need to ignore certain items in a get request e.g. images
			if (!method.equalsIgnoreCase("GET")){
				isValidInspectionResource=true;
				return;
			}
		}
		// use the uri to see if inspection should take place
		{
			String uri = requestLine.getUri();
			if (uri.endsWith(".jpg")
				|| uri.endsWith(".gif")
				|| uri.endsWith(".png")) {
	
				isValidInspectionResource = false;
				return;
			}
			if (uri.endsWith(".css")
				|| uri.endsWith(".swf")
				|| uri.endsWith(".js")) {
	
				isValidInspectionResource = false;
				return;
			}
			
			if (uri.endsWith(".html") || uri.endsWith(".htm") || uri.endsWith(".php") || uri.endsWith(".asp") || uri.endsWith(".jsp") || uri.endsWith(".cfm") || uri.endsWith(".cfml")){
				isValidInspectionResource = true;
				return;				
			}

		}
		
		try{
			// try get file type from header info
			//System.out.print("** Getting file info from head for "+requestLine);
			String mimeType=findMimeType(requestLine);
			//System.out.println("** Mime type "+mimeType);
			if (mimeType !=null && mimeType.indexOf("text/html") != -1){
				isValidInspectionResource = true;
				return;
			} else {
				isValidInspectionResource = false;
				return;
			}
				
		}catch(UnsupportedMethodException e){
			// dont know the file type, dont inspect it
			isValidInspectionResource = false;
		}
	}
	
	/**
	 * Creates a connection to the resource to attempt to find its mime type
	 * by looking at its header response.
	 * @param requestLine
	 * @return
	 * @throws UnsupportedMethodException
	 */
	private String findMimeType(RequestLine requestLine) throws UnsupportedMethodException{
		Socket soc = null;
		BufferedInputStream serverInput = null;
		BufferedOutputStream serverOutput = null;
		try{
			// setup the network connections
			if (ProxyConfiguration.isProxyDefined())
				soc = new Socket(ProxyConfiguration.getExternalProxyHost(),ProxyConfiguration.getExternalProxyPort());
			else
				soc = new Socket(requestLine.getHost(), requestLine.getPort());

			serverOutput = new BufferedOutputStream(soc.getOutputStream());
			serverInput = new BufferedInputStream(soc.getInputStream());
			
			String normalRequestLine=requestLine.toString();
			int indexOfCharAfterMethod=normalRequestLine.indexOf(' ');
			String headRequestLine="HEAD "+normalRequestLine.substring(indexOfCharAfterMethod,normalRequestLine.length());
			
			serverOutput.write(headRequestLine.getBytes());	
			serverOutput.write("\r\n".getBytes());
			serverOutput.flush();
			
			RequestPayload requestPayload=new RequestPayload(serverInput);
			RequestHeaders headers=requestPayload.getRequestHeaders();
			String mimeType=headers.getHeaderValue("Content-Type");
			return mimeType;
		} catch (Exception e){
			throw new UnsupportedMethodException();
		} finally{
			// close used network connections
			try {
				if (serverInput!=null)		
					serverInput.close();
				if (serverOutput!=null)
					serverOutput.close();		
				if (soc!=null)
					soc.close();
			} catch (Exception e){
				e.printStackTrace();
			}		
		}

	}

	/**
	 * Allows a decision to be based around intercepting external resource requests
	 * @return false is the requested resource is not valid for inspection, otherwise true
	 */
	public boolean isResourceToInspect() {
		return isValidInspectionResource;
	}

}
